"""
PyAEZ
Written by N. Lakmal Deshapriya
"""

import numpy as np
from scipy.interpolate import interp1d
import gdal

class UtilitiesCalc(object):

    def interpMonthlyToDaily(self, monthly_vector, cycle_begin, cycle_end, no_minus_values=False):
        # if 'no_minus_values' is true, negative values will be forced to be zero.

        doy_middle_of_month = np.arange(0,12)*30 + 15 # Calculate doy of middle of month

        #daily_vector = np.interp(np.arange(cycle_begin,cycle_end+1), doy_middle_of_month, monthly_vector)

        inter_fun = interp1d(doy_middle_of_month, monthly_vector, kind='quadratic', fill_value='extrapolate')
        daily_vector = inter_fun( np.arange(cycle_begin,cycle_end+1) )

        if no_minus_values:
            daily_vector[daily_vector<0] = 0

        return daily_vector

    def averageDailyToMonthly(self, daily_vector):

        monthly_vector = np.zeros(12)

        monthly_vector[0] = np.sum(daily_vector[:31])/31
        monthly_vector[1] = np.sum(daily_vector[31:59])/28
        monthly_vector[2] = np.sum(daily_vector[59:90])/31
        monthly_vector[3] = np.sum(daily_vector[90:120])/30
        monthly_vector[4] = np.sum(daily_vector[120:151])/31
        monthly_vector[5] = np.sum(daily_vector[151:181])/30
        monthly_vector[6] = np.sum(daily_vector[181:212])/31
        monthly_vector[7] = np.sum(daily_vector[212:243])/31
        monthly_vector[8] = np.sum(daily_vector[243:273])/30
        monthly_vector[9] = np.sum(daily_vector[273:304])/31
        monthly_vector[10] = np.sum(daily_vector[304:334])/30
        monthly_vector[11] = np.sum(daily_vector[334:])/31

        return monthly_vector

    def generateLatitudeMap(self, lat_min, lat_max, im_height, im_width):

        lat_lim = np.linspace(lat_min, lat_max, im_height);
        lon_lim = np.linspace(1, 1, im_width); # just temporary lon values, will not affect output of this function.
        [X_map,Y_map] = np.meshgrid(lon_lim,lat_lim);
        lat_map = np.flipud(Y_map);

        return lat_map

    def classifyFinalYield(self, est_yield):

        ''' Classifying Final Yield Map
        class 5 = very suitable = yields are equivalent to 80% or more of the overall maximum yield,
        class 4 = suitable = yields between 60% and 80%,
        class 3 = moderately suitable = yields between 40% and 60%,
        class 2 = marginally suitable = yields between 20% and 40%,
        class 1 = not suitable = yields between 0% and 20%.
        '''

        est_yield_max = np.amax( est_yield[est_yield>0] )
        est_yield_min = np.amin( est_yield[est_yield>0] )

        est_yield_20P = (est_yield_max-est_yield_min)*(20/100) + est_yield_min
        est_yield_40P = (est_yield_max-est_yield_min)*(40/100) + est_yield_min
        est_yield_60P = (est_yield_max-est_yield_min)*(60/100) + est_yield_min
        est_yield_80P = (est_yield_max-est_yield_min)*(80/100) + est_yield_min

        est_yield_class = np.zeros(est_yield.shape)

        est_yield_class[ np.all([0<est_yield, est_yield<=est_yield_20P], axis=0) ] = 1 # not suitable
        est_yield_class[ np.all([est_yield_20P<est_yield, est_yield<=est_yield_40P], axis=0) ] = 2 # marginally suitable
        est_yield_class[ np.all([est_yield_40P<est_yield, est_yield<=est_yield_60P], axis=0) ] = 3 # moderately suitable
        est_yield_class[ np.all([est_yield_60P<est_yield, est_yield<=est_yield_80P], axis=0) ] = 4 # suitable
        est_yield_class[ np.all([est_yield_80P<est_yield], axis=0)] = 5 # very suitable

        return est_yield_class

    def saveRaster(self, ref_raster_path, out_path, numpy_raster):

        # Read random image to get projection data
        img = gdal.Open(ref_raster_path)
        # allocating space in hard drive
        driver = gdal.GetDriverByName("GTiff")
        outdata = driver.Create(out_path, img.RasterXSize, img.RasterYSize, 1, gdal.GDT_Float32)
        # set image paramenters (imfrormation related to cordinates)
        outdata.SetGeoTransform(img.GetGeoTransform())
        outdata.SetProjection(img.GetProjection())
        # write numpy matrix as new band and set no data value for the band
        outdata.GetRasterBand(1).WriteArray(numpy_raster)
        outdata.GetRasterBand(1).SetNoDataValue(0)
        # flush data from memory to hard drive
        outdata.FlushCache()
        outdata=None

    def averageRasters(self, raster_3d):
        # input should be a 3D raster and averaging will be done through last dimension (usually corresponding to years)
        return np.sum(raster_3d, axis=2)/raster_3d.shape[-1]

    def windSpeedAt2m(self, wind_speed, altitude):
        # this function converts wind speed from a particular altitude to wind speed at 2m altitude. wind_speed can be a numpy array (can be 1D, 2D or 3D)
        return wind_speed * (4.87/np.log(67.8*altitude-5.42))



"""
PyAEZ
Written by N. Lakmal Deshapriya
"""

import numpy as np
import matplotlib.pyplot as plt
import imageio
import gdal

import UtilitiesCalc
import BioMassCalc
import ETOCalc
import CropWatCalc
import ThermalScreening

class CropSimulation(object):

    def __init__(self):

        self.set_mask = False

        self.set_tclimate_screening = False
        self.set_lgpt_screening = False
        self.set_Tsum_screening = False
        self.set_Tprofile_screening = False

    def setMonthlyClimateData(self, min_temp, max_temp, precipitation, short_rad, wind_speed, rel_humidity):
        # either one of monthly or daily climate data should be set
        self.minT_monthly = min_temp
        self.maxT_monthly = max_temp
        self.totalPrec_monthly = precipitation
        self.shortRad_monthly = short_rad
        self.wind2m_monthly = wind_speed
        self.rel_humidity_monthly = rel_humidity

        self.set_monthly = True

    def setDailyClimateData(self, min_temp, max_temp, precipitation, short_rad, wind_speed, rel_humidity):
        # either one of monthly or daily climate data should be set
        self.minT_daily = min_temp
        self.maxT_daily = max_temp
        self.totalPrec_daily = precipitation
        self.shortRad_daily = short_rad
        self.wind2m_daily = wind_speed
        self.rel_humidity_daily = rel_humidity

        self.set_monthly = False

    def setLocationTerrainData(self, lat_min, lat_max, elevation):
        self.elevation = elevation
        self.im_height = elevation.shape[0]
        self.im_width = elevation.shape[1]
        self.latitude_map = UtilitiesCalc.UtilitiesCalc().generateLatitudeMap(lat_min, lat_max, self.im_height, self.im_width)

    def setCropParameters(self, LAI, HI, legume, adaptability, cycle_len, D1, D2):
        self.LAi = LAI # leaf area index
        self.HI = HI # harvest index
        self.legume = legume # binary value
        self.adaptability = adaptability  #one of [1,2,3,4] classes
        self.cycle_len = cycle_len  #length of growing period

        self.D1 = D1  # rooting depth 1 (m)
        self.D2 = D2  # rooting depth 2 (m)

    def setCropCycleParameters(self, stage_per, kc, kc_all, yloss_f, yloss_f_all):
        self.d_per = stage_per # Percentage for D1, D2, D3, D4 stages
        self.kc = kc # 3 crop water requirements for initial, reproductive, the end of the maturation stages
        self.kc_all = kc_all # crop water requirements for entire growth cycle
        self.yloss_f = yloss_f  # yield loss for D1, D2, D3, D4
        self.yloss_f_all = yloss_f_all  # yield loss for entire growth cycle

    def setSoilWaterParameters(self, Sa, pc):
        self.Sa = Sa  # available soil moisture holding capacity (mm/m) , assumption
        self.pc = pc  # soil water depletion fraction below which ETa < ETo (from literature)

    '''All of bellow settings are optional'''

    # set mask of study area, this is optional
    def setStudyAreaMask(self, admin_mask, no_data_value):
        self.im_mask = admin_mask
        self.nodata_val = no_data_value

        self.set_mask = True

    def adjustForPerennialCrop(self,  aLAI, bLAI, aHI, bHI):
        self.LAi = self.LAi * ((self.cycle_len-aLAI)/bLAI) # leaf area index adjustment for perennial crops
        self.HI = self.HI * ((self.cycle_len-aHI)/bHI) # harvest index adjustment for perennial crops

    def setThermalClimateScreening(self, t_climate, no_t_climate):
        self.t_climate = t_climate
        self.no_t_climate = no_t_climate # list of unsuitable thermal climate

        self.set_tclimate_screening = True

    # set suitability screening, this is also optional
    def setLGPTScreening(self, no_lgpt, optm_lgpt):
        self.no_lgpt = no_lgpt
        self.optm_lgpt = optm_lgpt

        self.set_lgpt_screening = True

    def setTSumScreening(self, no_Tsum, optm_Tsum):
        self.no_Tsum = no_Tsum
        self.optm_Tsum = optm_Tsum

        self.set_Tsum_screening = True

    def setTProfileScreening(self, no_Tprofile, optm_Tprofile):
        self.no_Tprofile = no_Tprofile
        self.optm_Tprofile = optm_Tprofile

        self.set_Tprofile_screening = True

    def simulateCropCycle(self, start_doy=1, end_doy=365, step_doy=1, leap_year=False):

        # just a counter to keep track of progress
        count_pixel_completed = 0

        # this stores final result
        self.final_yield_rainfed = np.zeros((self.im_height, self.im_width));
        self.final_yield_irrig = np.zeros((self.im_height, self.im_width));
        self.crop_calender = np.zeros((self.im_height, self.im_width));

        for i_row in range(self.im_height):

            if self.set_mask:
                print('Done: ' + str(int((count_pixel_completed/np.sum(self.im_mask!=self.nodata_val))*100)) + ' %')
            else:
                print('Done: ' + str(int((count_pixel_completed/(self.im_height*self.im_width))*100)) + ' %')

            for i_col in range(self.im_width):

                # check current location (pixel) is outside of study area or not. if it's outside of study area goes to next location (pixel)
                if self.set_mask:
                    if self.im_mask[i_row, i_col] == self.nodata_val:
                        continue
                count_pixel_completed = count_pixel_completed + 1

                # this allows handing leap and non-leap year differently. This is only relevant for monthly data because this value will be used in interpolations.
                # In case of daily data, length of vector will be taken as number of days in  a year.
                if leap_year:
                    days_in_year = 366
                else:
                    days_in_year = 365

                # extract climate data for particular location. And if climate data are monthly data, they are interpolated as daily data
                if self.set_monthly:
                    obj_utilities = UtilitiesCalc.UtilitiesCalc()

                    minT_daily_point = obj_utilities.interpMonthlyToDaily(self.minT_monthly[i_row, i_col,:], 1, days_in_year)
                    maxT_daily_point = obj_utilities.interpMonthlyToDaily(self.maxT_monthly[i_row, i_col,:], 1, days_in_year)
                    shortRad_daily_point = obj_utilities.interpMonthlyToDaily(self.shortRad_monthly[i_row, i_col,:],  1, days_in_year, no_minus_values=True)
                    wind2m_daily_point = obj_utilities.interpMonthlyToDaily(self.wind2m_monthly[i_row, i_col,:],  1, days_in_year, no_minus_values=True)
                    totalPrec_daily_point = obj_utilities.interpMonthlyToDaily(self.totalPrec_monthly[i_row, i_col,:],  1, days_in_year, no_minus_values=True)
                    rel_humidity_daily_point = obj_utilities.interpMonthlyToDaily(self.rel_humidity_monthly[i_row, i_col,:],  1, days_in_year, no_minus_values=True)
                else:
                    minT_daily_point = self.minT_daily[i_row, i_col,:]
                    maxT_daily_point = self.maxT_daily[i_row, i_col,:]
                    shortRad_daily_point = self.shortRad_daily[i_row, i_col,:]
                    wind2m_daily_point = self.wind2m_daily[i_row, i_col,:]
                    totalPrec_daily_point = self.totalPrec_daily[i_row, i_col,:]
                    rel_humidity_daily_point = self.rel_humidity_daily[i_row, i_col,:]

                # calculate ETO for full year for particular location (pixel)
                obj_eto = ETOCalc.ETOCalc(1, minT_daily_point.shape[0], self.latitude_map[i_row, i_col], self.elevation[i_row, i_col])
                shortRad_dailyy_point_MJm2day = (shortRad_daily_point*3600*24)/1000000 # convert w/m2 to MJ/m2/day
                obj_eto.setClimateData(minT_daily_point, maxT_daily_point, wind2m_daily_point, shortRad_dailyy_point_MJm2day, rel_humidity_daily_point)
                pet_daily_point = obj_eto.calculateETO()

                # list that stores yield estimations of all cycles per particular location (pixel)
                yield_of_all_crop_cycles_rainfed = []
                yield_of_all_crop_cycles_irrig = []

                for i_cycle in range(start_doy, end_doy+1, step_doy):

                    # just repeat data for year 2 times, to simulate for a entire year. just for computational convenient
                    minT_daily_2year = np.tile(minT_daily_point, 2)
                    maxT_daily_2year = np.tile(maxT_daily_point, 2)
                    shortRad_daily_2year = np.tile(shortRad_daily_point, 2)
                    wind2m_daily_2year = np.tile(wind2m_daily_point, 2)
                    totalPrec_daily_2year = np.tile(totalPrec_daily_point, 2)
                    pet_daily_2year = np.tile(pet_daily_point, 2)

                    # extract climate data within the season to pass in to calculation classes
                    minT_daily_season = minT_daily_2year[i_cycle : i_cycle+self.cycle_len]
                    maxT_daily_season = maxT_daily_2year[i_cycle : i_cycle+self.cycle_len]
                    shortRad_daily_season = shortRad_daily_2year[i_cycle : i_cycle+self.cycle_len]
                    wind2m_daily_season = wind2m_daily_2year[i_cycle : i_cycle+self.cycle_len]
                    totalPrec_daily_season = totalPrec_daily_2year[i_cycle : i_cycle+self.cycle_len]
                    pet_daily_season = pet_daily_2year[i_cycle : i_cycle+self.cycle_len]

                    # conduct tests to check simulation should be carried out or not based on growing period threshold. if not, goes to next location (pixel)
                    obj_screening = ThermalScreening.ThermalScreening()
                    obj_screening.setClimateData(minT_daily_season, maxT_daily_season)

                    if self.set_tclimate_screening:
                        obj_screening.setThermalClimateScreening(self.t_climate[i_row, i_col], self.no_t_climate)
                    if self.set_lgpt_screening:
                        obj_screening.setLGPTScreening(self.no_lgpt, self.optm_lgpt)
                    if self.set_Tsum_screening:
                        obj_screening.setTSumScreening(self.no_Tsum, self.optm_Tsum)
                    if self.set_Tprofile_screening:
                        obj_screening.setTProfileScreening(self.no_Tprofile, self.optm_Tprofile)

                    thermal_screening_f = 1
                    if not obj_screening.getSuitability():
                        continue
                    else:
                        thermal_screening_f = obj_screening.getReductionFactor()

                    # calculate biomass
                    obj_maxyield = BioMassCalc.BioMassCalc(i_cycle, i_cycle+self.cycle_len-1, self.latitude_map[i_row, i_col])
                    obj_maxyield.setClimateData(minT_daily_season, maxT_daily_season, shortRad_daily_season)
                    obj_maxyield.setCropParameters(self.LAi, self.HI, self.legume, self.adaptability)
                    obj_maxyield.calculateBioMass()
                    est_yield = obj_maxyield.calculateYield()

                    # reduce thermal screening factor
                    est_yield = est_yield * thermal_screening_f

                    # apply cropwat
                    obj_cropwat = CropWatCalc.CropWatCalc(i_cycle, i_cycle+self.cycle_len-1)
                    obj_cropwat.setClimateData(pet_daily_season, totalPrec_daily_season)
                    # check Sa is a raster or single value and extract Sa value accordingly
                    if len(np.array(self.Sa).shape) == 2:
                        Sa_temp = self.Sa[i_row, i_col]
                    else:
                        Sa_temp = self.Sa
                    obj_cropwat.setCropParameters(self.d_per, self.kc, self.kc_all, self.yloss_f, self.yloss_f_all, est_yield, self.D1, self.D2, Sa_temp, self.pc)
                    est_yield_moisture_limited = obj_cropwat.calculateMoistureLimitedYield()

                    # append current cycle yield to a list
                    yield_of_all_crop_cycles_rainfed.append( est_yield_moisture_limited )
                    yield_of_all_crop_cycles_irrig.append( est_yield )

                    # get maximum yield from all simulation for a particular location (pixel) and assign to final map
                    if len(yield_of_all_crop_cycles_irrig) > 0:
                        self.final_yield_rainfed[i_row, i_col] = np.max(yield_of_all_crop_cycles_rainfed)
                        self.final_yield_irrig[i_row, i_col] = np.max(yield_of_all_crop_cycles_irrig)
                        self.crop_calender[i_row, i_col] = np.where(yield_of_all_crop_cycles_rainfed==np.max(yield_of_all_crop_cycles_rainfed))[0][0] * step_doy

        print('Simulations Completed !')

    def getEstimatedYieldRainfed(self):
        return self.final_yield_rainfed

    def getEstimatedYieldIrrigated(self):
        return self.final_yield_irrig

    def getOptimumCycleStartDate(self):
        return self.crop_calender



"""
PyAEZ
Written by N. Lakmal Deshapriya
"""

import numpy as np

import ALL_REDUCTION_FACTORS_IRR as crop_P_IRR
import ALL_REDUCTION_FACTORS_RAIN as crop_P_RAIN

class ClimaticConstraints(object):

    def applyClimaticConstraints(self, lgp_eq, yield_in, irr_or_rain):

        if irr_or_rain == 'I':
            crop_P = crop_P_IRR
        elif irr_or_rain == 'R':
            crop_P = crop_P_RAIN

        class_break = np.array(crop_P.lgp_eq_class)
        reduction_fact = np.array(crop_P.lgp_eq_red_fr)

        yield_final = np.copy(yield_in)
        lgp_eq = np.round(lgp_eq)
        lgp_eq[np.isnan(lgp_eq)] = -99 # This suppresses warning with NaN values

        '''min is replaced by product based on GAEZ doc, still keeping commented min code also just in case'''
        # min_yield_fact = np.min(reduction_fact, axis=0) / 100
        min_yield_fact = np.prod(reduction_fact/100, axis=0)

        for class_num in range(class_break.shape[0]):
            temp_idx = np.logical_and(class_break[class_num,0]<=lgp_eq, lgp_eq<=class_break[class_num,1])
            yield_final[temp_idx] = yield_in[temp_idx] * min_yield_fact[class_num]

        return yield_final



"""
PyAEZ
Written by N. Lakmal Deshapriya and Thaileng Thol
"""

import numpy as np
np.seterr(divide='ignore', invalid='ignore') # ignore "divide by zero" or "divide by NaN" warning
import UtilitiesCalc
import ETOCalc

class ClimateRegime(object):

    # set mask of study area, this is optional
    def setStudyAreaMask(self, admin_mask, no_data_value):
        self.im_mask = admin_mask
        self.nodata_val = no_data_value

        self.set_mask = True

    def setLocationTerrainData(self, lat_min, lat_max, elevation):
        self.elevation = elevation
        self.im_height = elevation.shape[0]
        self.im_width = elevation.shape[1]
        self.latitude = UtilitiesCalc.UtilitiesCalc().generateLatitudeMap(lat_min, lat_max, self.im_height, self.im_width)

    def setMonthlyClimateData(self, min_temp, max_temp, precipitation, short_rad, wind_speed, rel_humidity):

        self.meanT_daily = np.zeros((self.im_height, self.im_width, 365))
        self.totalPrec_daily = np.zeros((self.im_height, self.im_width, 365))
        self.pet_daily = np.zeros((self.im_height, self.im_width, 365))

        # Interpolate monthly to daily data
        obj_utilities = UtilitiesCalc.UtilitiesCalc()

        meanT_monthly = (min_temp+max_temp)/2

        for i_row in range(self.im_height):
            for i_col in range(self.im_width):

                if self.set_mask:
                    if self.im_mask[i_row, i_col] == self.nodata_val:
                        continue

                self.meanT_daily[i_row, i_col, :] = obj_utilities.interpMonthlyToDaily(meanT_monthly[i_row, i_col,:], 1, 365)
                self.totalPrec_daily[i_row, i_col, :] = obj_utilities.interpMonthlyToDaily(precipitation[i_row, i_col,:], 1, 365, no_minus_values=True)

                minT_daily = obj_utilities.interpMonthlyToDaily(min_temp[i_row, i_col,:], 1, 365)
                maxT_daily = obj_utilities.interpMonthlyToDaily(max_temp[i_row, i_col,:], 1, 365)
                radiation_daily = obj_utilities.interpMonthlyToDaily(short_rad[i_row, i_col,:], 1, 365, no_minus_values=True)
                wind_daily = obj_utilities.interpMonthlyToDaily(wind_speed[i_row, i_col,:], 1, 365, no_minus_values=True)
                rel_humidity_daily = obj_utilities.interpMonthlyToDaily(rel_humidity[i_row, i_col,:], 1, 365, no_minus_values=True)

                # calculation of ET
                obj_eto = ETOCalc.ETOCalc(1, 365, self.latitude[i_row, i_col], self.elevation[i_row, i_col])
                shortrad_daily_MJm2day = (radiation_daily*3600*24)/1000000 # convert w/m2 to MJ/m2/day
                obj_eto.setClimateData(minT_daily, maxT_daily, wind_daily, shortrad_daily_MJm2day, rel_humidity_daily)
                self.pet_daily[i_row, i_col, :] = obj_eto.calculateETO()

        # sea level temperature
        self.meanT_daily_sealevel = self.meanT_daily + np.tile(np.reshape(self.elevation/100*0.55, (self.im_height,self.im_width,1)), (1,1,365))

        # P over PET ratio
        self.P_by_PET_daily = self.totalPrec_daily / self.pet_daily

    def setDailyClimateData(self, min_temp, max_temp, precipitation, short_rad, wind_speed, rel_humidity):

        self.meanT_daily = np.zeros((self.im_height, self.im_width, 365))
        self.totalPrec_daily = np.zeros((self.im_height, self.im_width, 365))
        self.pet_daily = np.zeros((self.im_height, self.im_width, 365))

        # calculation of ET
        for i_row in range(self.im_height):
            for i_col in range(self.im_width):

                if self.set_mask:
                    if self.im_mask[i_row, i_col] == self.nodata_val:
                        continue

                self.meanT_daily[i_row, i_col, :] = (min_temp[i_row, i_col, :]+max_temp[i_row, i_col, :])/2
                self.totalPrec_daily[i_row, i_col, :] = precipitation[i_row, i_col, :]

                obj_eto = ETOCalc.ETOCalc(1, 365, self.latitude[i_row, i_col], self.elevation[i_row, i_col])
                shortrad_daily_MJm2day = (short_rad[i_row, i_col, :]*3600*24)/1000000 # convert w/m2 to MJ/m2/day
                obj_eto.setClimateData(min_temp[i_row, i_col, :], max_temp[i_row, i_col, :], wind_speed[i_row, i_col, :], shortrad_daily_MJm2day, rel_humidity[i_row, i_col, :])
                self.pet_daily[i_row, i_col, :] = obj_eto.calculateETO()

        # sea level temperature
        self.meanT_daily_sealevel = self.meanT_daily + np.tile(np.reshape(self.elevation/100*0.55, (self.im_height,self.im_width,1)), (1,1,365))

        # P over PET ratio
        self.P_by_PET_daily = self.totalPrec_daily / self.pet_daily

    def getThermalClimate(self):

        thermal_climate = np.zeros((self.im_height, self.im_width))

        for i_r in range(self.im_height):
            for i_c in range(self.im_width):

                if self.set_mask:
                    if self.im_mask[i_r, i_c] == self.nodata_val:
                        continue

                # converting daily to monthly
                obj_utilities = UtilitiesCalc.UtilitiesCalc()
                meanT_monthly_sealevel_v = obj_utilities.averageDailyToMonthly(self.meanT_daily_sealevel[i_r,i_c,:])
                meanT_monthly_v = obj_utilities.averageDailyToMonthly(self.meanT_daily[i_r,i_c,:])
                P_by_PET_monthly_v = obj_utilities.averageDailyToMonthly(self.P_by_PET_daily[i_r,i_c,:])

                if np.min(meanT_monthly_sealevel_v) >= 18:
                    # Tropics
                    #if np.mean(meanT_monthly_v) > 20:
                    if np.min(meanT_monthly_v) > 20:
                        # Tropical lowland
                        thermal_climate[i_r,i_c] = 1
                    else:
                        # Tropical highland
                        thermal_climate[i_r,i_c] = 2

                elif np.min(meanT_monthly_sealevel_v) >= 5 and np.sum(meanT_monthly_sealevel_v>=10) >= 8:
                    # SubTropic
                    if np.sum(self.totalPrec_daily[i_r,i_c,:]) < 250:
                        # 'Subtropics Low Rainfall
                        thermal_climate[i_r,i_c] = 3
                    elif self.latitude[i_r,i_c]>=0 and np.mean(P_by_PET_monthly_v[3:9]) >= np.mean([P_by_PET_monthly_v[9:12],P_by_PET_monthly_v[0:3]]):
                        # Subtropics Summer Rainfall
                        thermal_climate[i_r,i_c] = 4
                    elif self.latitude[i_r,i_c]<0 and np.mean(P_by_PET_monthly_v[3:9]) <= np.mean([P_by_PET_monthly_v[9:12],P_by_PET_monthly_v[0:3]]):
                        # Subtropics Summer Rainfall
                        thermal_climate[i_r,i_c] = 4
                    elif self.latitude[i_r,i_c]>=0 and np.mean(P_by_PET_monthly_v[3:9]) <= np.mean([P_by_PET_monthly_v[9:12],P_by_PET_monthly_v[0:3]]):
                        # Subtropics Winter Rainfall
                        thermal_climate[i_r,i_c] = 5
                    elif self.latitude[i_r,i_c]<0 and np.mean(P_by_PET_monthly_v[3:9]) >= np.mean([P_by_PET_monthly_v[9:12],P_by_PET_monthly_v[0:3]]):
                        # Subtropics Winter Rainfall
                        thermal_climate[i_r,i_c] = 5

                elif np.sum(meanT_monthly_sealevel_v>=10) >= 4:
                    # Temperate
                    if np.max(meanT_monthly_v)-np.min(meanT_monthly_v) < 20:
                        # Oceanic Temperate
                        thermal_climate[i_r,i_c] = 6
                    elif np.max(meanT_monthly_v)-np.min(meanT_monthly_v) < 35:
                        # Sub-Continental Temperate
                        thermal_climate[i_r,i_c] = 7
                    else:
                        # Continental Temperate
                        thermal_climate[i_r,i_c] = 8

                elif np.sum(meanT_monthly_sealevel_v>=10) >= 1:
                    # Boreal
                    if np.max(meanT_monthly_v)-np.min(meanT_monthly_v) < 20:
                        # Oceanic Boreal
                        thermal_climate[i_r,i_c] = 9
                    elif np.max(meanT_monthly_v)-np.min(meanT_monthly_v) < 35:
                        # Sub-Continental Boreal
                        thermal_climate[i_r,i_c] = 10
                    else:
                        # Continental Boreal
                        thermal_climate[i_r,i_c] = 11
                else:
                    # Arctic
                    thermal_climate[i_r,i_c] = 12

        return thermal_climate

    def getThermalZone(self):

        thermal_zone = np.zeros((self.im_height, self.im_width))

        for i_r in range(self.im_height):
            for i_c in range(self.im_width):

                if self.set_mask:
                    if self.im_mask[i_r, i_c] == self.nodata_val:
                        continue

                # converting daily to monthly
                obj_utilities = UtilitiesCalc.UtilitiesCalc()
                meanT_monthly_sealevel_v = obj_utilities.averageDailyToMonthly(self.meanT_daily_sealevel[i_r,i_c,:])
                meanT_monthly_v = obj_utilities.averageDailyToMonthly(self.meanT_daily[i_r,i_c,:])

                if np.min(meanT_monthly_sealevel_v) >= 18:
                    # Tropics
                    if np.mean(meanT_monthly_v) > 20:
                        # Warm
                        thermal_zone[i_r,i_c] = 1
                    elif np.sum(meanT_monthly_v<18) >= 1 and np.min(meanT_monthly_v) > 5 and np.sum(meanT_monthly_v>10) >= 8:
                        # Moderately Cool
                        thermal_zone[i_r,i_c] = 2
                    elif np.sum(meanT_monthly_v<5) >= 1 and np.sum(meanT_monthly_v>10) >= 4:
                        # Cool
                        thermal_zone[i_r,i_c] = 3
                    elif np.sum(meanT_monthly_v<5) >= 1 and np.sum(meanT_monthly_v>10) >= 1:
                        # Cold
                        thermal_zone[i_r,i_c] = 4
                    elif np.sum(meanT_monthly_v<10) == 12:
                        # Very Cold
                        thermal_zone[i_r,i_c] = 5

                elif np.min(meanT_monthly_sealevel_v) >= 5 and np.sum(meanT_monthly_sealevel_v>=10) >= 8:
                    # SubTropic
                    if np.mean(meanT_monthly_v) > 20:
                        # Warm
                        thermal_zone[i_r,i_c] = 6
                    elif np.sum(meanT_monthly_v<18) >= 1 and np.min(meanT_monthly_v) > 5 and np.sum(meanT_monthly_v>10) >= 8:
                        # Moderately Cool
                        thermal_zone[i_r,i_c] = 7
                    elif np.sum(meanT_monthly_v<5) >= 1 and np.sum(meanT_monthly_v>10) >= 4:
                        # Cool
                        thermal_zone[i_r,i_c] = 8
                    elif np.sum(meanT_monthly_v<5) >= 1 and np.sum(meanT_monthly_v>10) >= 1:
                        # Cold
                        thermal_zone[i_r,i_c] = 9
                    elif np.sum(meanT_monthly_v<10) == 12:
                        # Very Cold
                        thermal_zone[i_r,i_c] = 10

                elif np.sum(meanT_monthly_sealevel_v>=10) >= 4:
                    # Temperate
                    if np.mean(meanT_monthly_v) > 20:
                        # Warm
                        thermal_zone[i_r,i_c] = 11
                    elif np.sum(meanT_monthly_v<18) >= 1 and np.min(meanT_monthly_v) > 5 and np.sum(meanT_monthly_v>10) >= 8:
                        # Moderately Cool
                        thermal_zone[i_r,i_c] = 12
                    elif np.sum(meanT_monthly_v<5) >= 1 and np.sum(meanT_monthly_v>10) >= 4:
                        # Cool
                        thermal_zone[i_r,i_c] = 13
                    elif np.sum(meanT_monthly_v<5) >= 1 and np.sum(meanT_monthly_v>10) >= 1:
                        # Cold
                        thermal_zone[i_r,i_c] = 14
                    elif np.sum(meanT_monthly_v<10) == 12:
                        # Very Cold
                        thermal_zone[i_r,i_c] = 15

                elif np.sum(meanT_monthly_sealevel_v>=10) >= 1:
                    # Boreal
                    if np.mean(meanT_monthly_v) > 20:
                        # Warm
                        thermal_zone[i_r,i_c] = 16
                    elif np.sum(meanT_monthly_v<18) >= 1 and np.min(meanT_monthly_v) > 5 and np.sum(meanT_monthly_v>10) >= 8:
                        # Moderately Cool
                        thermal_zone[i_r,i_c] = 17
                    elif np.sum(meanT_monthly_v<5) >= 1 and np.sum(meanT_monthly_v>10) >= 4:
                        # Cool
                        thermal_zone[i_r,i_c] = 18
                    elif np.sum(meanT_monthly_v<5) >= 1 and np.sum(meanT_monthly_v>10) >= 1:
                        # Cold
                        thermal_zone[i_r,i_c] = 19
                    elif np.sum(meanT_monthly_v<10) == 12:
                        # Very Cold
                        thermal_zone[i_r,i_c] = 20
                else:
                    # Arctic
                    thermal_zone[i_r,i_c] = 21

        return thermal_zone

    def getThermalLGP0(self):
        return np.sum(self.meanT_daily>0, axis=2)

    def getThermalLGP5(self):
        return np.sum(self.meanT_daily>5, axis=2)

    def getThermalLGP10(self):
        return np.sum(self.meanT_daily>10, axis=2)

    def getTemperatureSum0(self):
        tempT = self.meanT_daily
        tempT[tempT<=0] = 0
        return np.sum(tempT, axis=2)

    def getTemperatureSum5(self):
        tempT = self.meanT_daily
        tempT[tempT<=5] = 0
        return np.sum(tempT, axis=2)

    def getTemperatureSum10(self):
        tempT = self.meanT_daily
        tempT[tempT<=10] = 0
        return np.sum(tempT, axis=2)

    def getTemperatureProfile(self):

        meanT_daily_add1day = np.concatenate((self.meanT_daily, self.meanT_daily[:,:,0:1]), axis=-1)
        meanT_first = meanT_daily_add1day[:,:,:-1]
        meanT_diff = meanT_daily_add1day[:,:,1:] - meanT_daily_add1day[:,:,:-1]

        A9 = np.sum( np.logical_and(meanT_diff>0, meanT_first<-5), axis=2 )
        A8 = np.sum( np.logical_and(meanT_diff>0, np.logical_and(meanT_first>=-5, meanT_first<0)), axis=2 )
        A7 = np.sum( np.logical_and(meanT_diff>0, np.logical_and(meanT_first>=0, meanT_first<5)), axis=2 )
        A6 = np.sum( np.logical_and(meanT_diff>0, np.logical_and(meanT_first>=5, meanT_first<10)), axis=2 )
        A5 = np.sum( np.logical_and(meanT_diff>0, np.logical_and(meanT_first>=10, meanT_first<15)), axis=2 )
        A4 = np.sum( np.logical_and(meanT_diff>0, np.logical_and(meanT_first>=15, meanT_first<20)), axis=2 )
        A3 = np.sum( np.logical_and(meanT_diff>0, np.logical_and(meanT_first>=20, meanT_first<25)), axis=2 )
        A2 = np.sum( np.logical_and(meanT_diff>0, np.logical_and(meanT_first>=25, meanT_first<30)), axis=2 )
        A1 = np.sum( np.logical_and(meanT_diff>0, meanT_first>=30), axis=2 )

        B9 = np.sum( np.logical_and(meanT_diff<0, meanT_first<-5), axis=2 )
        B8 = np.sum( np.logical_and(meanT_diff<0, np.logical_and(meanT_first>=-5, meanT_first<0)), axis=2 )
        B7 = np.sum( np.logical_and(meanT_diff<0, np.logical_and(meanT_first>=0, meanT_first<5)), axis=2 )
        B6 = np.sum( np.logical_and(meanT_diff<0, np.logical_and(meanT_first>=5, meanT_first<10)), axis=2 )
        B5 = np.sum( np.logical_and(meanT_diff<0, np.logical_and(meanT_first>=10, meanT_first<15)), axis=2 )
        B4 = np.sum( np.logical_and(meanT_diff<0, np.logical_and(meanT_first>=15, meanT_first<20)), axis=2 )
        B3 = np.sum( np.logical_and(meanT_diff<0, np.logical_and(meanT_first>=20, meanT_first<25)), axis=2 )
        B2 = np.sum( np.logical_and(meanT_diff<0, np.logical_and(meanT_first>=25, meanT_first<30)), axis=2 )
        B1 = np.sum( np.logical_and(meanT_diff<0, meanT_first>=30), axis=2 )

        return [A9,A8,A7,A6,A5,A4,A3,A2,A1,B1,B2,B3,B4,B5,B6,B7,B8,B9]


    def getLGP(self, Sa = 100, pc = 0.5, kc = 1, D = 1):

        # Sa: available soil moisture holding capacity (mm/m) , usually assume as 100
        # kc: crop water requirements for entire growth cycle
        # D: rooting depth (m)
        # pc: soil water depletion fraction below which ETa < ETo (from literature)

        petc = np.zeros(self.pet_daily.shape)
        peta = np.zeros(self.pet_daily.shape)
        petc = self.pet_daily * 1

        for i_r in range(self.im_height):
            for i_c in range(self.im_width):

                if self.set_mask:
                    if self.im_mask[i_r, i_c] == self.nodata_val:
                        continue

                if len(np.array(Sa).shape) == 2:
                    Sa_temp = Sa[i_r, i_c]
                else:
                    Sa_temp = Sa

                W = 0

                for ii in range(self.pet_daily.shape[-1]):
                    if self.totalPrec_daily[i_r,i_c,ii] >= petc[i_r,i_c,ii]:
                        peta[i_r,i_c,ii] = petc[i_r,i_c,ii]
                    elif self.totalPrec_daily[i_r,i_c,ii] + W >= Sa_temp*D*(1-pc):
                        peta[i_r,i_c,ii] = petc[i_r,i_c,ii]
                    else:
                        kk = (W+self.totalPrec_daily[i_r,i_c,ii]) / (Sa_temp*D*(1-pc))
                        peta[i_r,i_c,ii] = kk * petc[i_r,i_c,ii]

                    W = np.min([W+self.totalPrec_daily[i_r,i_c,ii]-peta[i_r,i_c,ii], Sa_temp*D])
                    if W<0: W=0

        return np.sum( (peta/self.pet_daily)>0.5, axis=2 )

    def getLGP_G_5_and_10(self):

        kc = 1 # crop water requirements for entire growth cycle
        Sa = 100 # available soil moisture holding capacity (mm/m) , usually assume as 100
        D = 1 # rooting depth (m)
        pc = 0.5 # soil water depletion fraction below which ETa < ETo (from literature)

        petc = np.zeros(self.pet_daily.shape)
        peta = np.zeros(self.pet_daily.shape)
        petc = self.pet_daily * 1

        for i_r in range(self.im_height):
            for i_c in range(self.im_width):

                if self.set_mask:
                    if self.im_mask[i_r, i_c] == self.nodata_val:
                        continue

                W = 0

                for ii in range(self.pet_daily.shape[-1]):
                    if self.totalPrec_daily[i_r,i_c,ii] >= petc[i_r,i_c,ii]:
                        peta[i_r,i_c,ii] = petc[i_r,i_c,ii]
                    elif self.totalPrec_daily[i_r,i_c,ii] + W >= Sa*D*(1-pc):
                        peta[i_r,i_c,ii] = petc[i_r,i_c,ii]
                    else:
                        kk = (W+self.totalPrec_daily[i_r,i_c,ii]) / (Sa*D*(1-pc))
                        peta[i_r,i_c,ii] = kk * petc[i_r,i_c,ii]

                    W = np.min([W+self.totalPrec_daily[i_r,i_c,ii]-peta[i_r,i_c,ii], Sa*D])
                    if W<0: W=0

        tempT = self.meanT_daily
        tempT[np.logical_and(tempT<5, (peta/self.pet_daily)<=0.5)] = 0
        ts_g_t5 = np.sum(tempT, axis=2)

        tempT = self.meanT_daily
        tempT[np.logical_and(tempT<10, (peta/self.pet_daily)<=0.5)] = 0
        ts_g_t10 = np.sum(tempT, axis=2)

        return [ts_g_t5, ts_g_t10]

    def getLGPClassified(self, lgp):

        lgp_class = np.zeros(lgp.shape)

        lgp_class[lgp>=365] = 7 # Per-humid
        lgp_class[np.logical_and(lgp>=270, lgp<365)] = 6 # Humid
        lgp_class[np.logical_and(lgp>=180, lgp<270)] = 5 # Sub-humid
        lgp_class[np.logical_and(lgp>=120, lgp<180)] = 4 # Moist semi-arid
        lgp_class[np.logical_and(lgp>=60, lgp<120)] = 3 # Dry semi-arid
        lgp_class[np.logical_and(lgp>0, lgp<60)] = 2 # Arid
        lgp_class[lgp<=0] = 1 # Hyper-arid

        return lgp_class

    def getLGPEquivalent(self):

        moisture_index = np.sum(self.totalPrec_daily, axis=2)/np.sum(self.pet_daily, axis=2)

        lgp_equv = 14.0 + 293.66*moisture_index - 61.25*moisture_index*moisture_index
        lgp_equv[ moisture_index > 2.4 ] = 366

        return lgp_equv


    def getMultiCroppingZones(self, t_climate, lgp, lgp_t5, lgp_t10, ts_t0, ts_t10):

        ts_g_t5_and_10 = self.getLGP_G_5_and_10()
        ts_g_t5 = ts_g_t5_and_10[0]
        ts_g_t10 = ts_g_t5_and_10[1]

        multi_cropping = np.zeros(lgp.shape)

        zone_B = np.all([lgp>=45, lgp_t5>=120, lgp_t10>=90, ts_t0>=1600, ts_t10>=1200], axis=0)
        zone_C1 = np.all([lgp>=220, lgp_t5>=220, lgp_t10>=120, ts_t0>=5500, ts_g_t5>=3200, ts_g_t10>=2700], axis=0)
        #zone_C2 = np.all([lgp>=200, lgp_t5>=210, lgp_t10>=120, ts_t0>=6400, ts_g_t5>=3200, ts_g_t10>=2700], axis=0)
        zone_C3 = np.all([lgp>=180, lgp_t5>=200, lgp_t10>=120, ts_t0>=7200, ts_g_t5>=3200, ts_g_t10>=2700], axis=0)
        zone_D1 = np.all([lgp>=270, lgp_t5>=270, lgp_t10>=165, ts_t0>=5500, ts_g_t5>=4000, ts_g_t10>=3200], axis=0)
        #zone_D2 = np.all([lgp>=240, lgp_t5>=240, lgp_t10>=165, ts_t0>=6400, ts_g_t5>=4000, ts_g_t10>=3200], axis=0)
        zone_D3 = np.all([lgp>=210, lgp_t5>=240, lgp_t10>=165, ts_t0>=7200, ts_g_t5>=4000, ts_g_t10>=3200], axis=0)
        zone_F = np.all([lgp>=300, lgp_t5>=300, lgp_t10>=240, ts_t0>=7200, ts_t10>=7000, ts_g_t5>=5100, ts_g_t10>=4800], axis=0)
        zone_H = np.all([lgp>=360, lgp_t5>=360, lgp_t10>=330, ts_t0>=7200, ts_t10>=7000], axis=0)

        multi_cropping[np.all([zone_B,t_climate<=2], axis=0)] = 2
        multi_cropping[np.all([zone_C1,t_climate==1], axis=0)] = 3
        multi_cropping[np.all([zone_C3,t_climate==2], axis=0)] = 3
        multi_cropping[np.all([zone_D1,t_climate==1], axis=0)] = 4
        multi_cropping[np.all([zone_D3,t_climate==2], axis=0)] = 4
        multi_cropping[np.all([zone_F,t_climate<=2], axis=0)] = 6
        multi_cropping[np.all([zone_H,t_climate<=2], axis=0)] = 8

        zone_B = np.all([lgp>=45, lgp_t5>=120, lgp_t10>=90, ts_t0>=1600, ts_t10>=1200], axis=0)
        zone_C = np.all([lgp>=180, lgp_t5>=200, lgp_t10>=120, ts_t0>=3600, ts_t10>=3000, ts_g_t5>=3200, ts_g_t10>=2700], axis=0)
        zone_D = np.all([lgp>=210, lgp_t5>=240, lgp_t10>=165, ts_t0>=4500, ts_t10>=3600, ts_g_t5>=4000, ts_g_t10>=3200], axis=0)
        zone_E = np.all([lgp>=240, lgp_t5>=270, lgp_t10>=180, ts_t0>=4800, ts_t10>=4500, ts_g_t5>=4300, ts_g_t10>=4000], axis=0)
        zone_F = np.all([lgp>=300, lgp_t5>=300, lgp_t10>=240, ts_t0>=5400, ts_t10>=5100, ts_g_t5>=5100, ts_g_t10>=4800], axis=0)
        zone_G = np.all([lgp>=330, lgp_t5>=330, lgp_t10>=270, ts_t0>=5700, ts_t10>=5500], axis=0)
        zone_H = np.all([lgp>=360, lgp_t5>=360, lgp_t10>=330, ts_t0>=7200, ts_t10>=7000], axis=0)

        multi_cropping[np.all([zone_B,t_climate>=3,t_climate<=8], axis=0)] = 2
        multi_cropping[np.all([zone_C,t_climate>=3,t_climate<=8], axis=0)] = 3
        multi_cropping[np.all([zone_D,t_climate>=3,t_climate<=8], axis=0)] = 4
        multi_cropping[np.all([zone_E,t_climate>=3,t_climate<=8], axis=0)] = 5
        multi_cropping[np.all([zone_F,t_climate>=3,t_climate<=8], axis=0)] = 6
        multi_cropping[np.all([zone_G,t_climate>=3,t_climate<=8], axis=0)] = 7
        multi_cropping[np.all([zone_H,t_climate>=3,t_climate<=8], axis=0)] = 8

        return multi_cropping



"""
PyAEZ
Written by Thaileng Thol
"""

import numpy as np
import numba as nb

class ETOCalc(object):

    def __init__(self, cycle_begin, cycle_end, latitude, altitude):
        self.cycle_begin = cycle_begin
        self.cycle_end = cycle_end
        self.latitude = latitude
        self.alt = altitude

    def setClimateData(self, min_temp, max_temp, wind_speed, short_rad, rel_humidity):

        self.minT_daily = min_temp # Celcius
        self.maxT_daily = max_temp # Celcius
        self.windspeed_daily = wind_speed # m/s at 2m
        self.shortRad_daily = short_rad # MJ/m2.day
        self.rel_humidity = rel_humidity # Fraction

    @staticmethod
    @nb.jit(nopython=True)
    def calculateETONumba(cycle_begin, cycle_end, latitude, alt,  minT_daily, maxT_daily, windspeed_daily, shortRad_daily, rel_humidity):
        # Monthly Average Temperature
        ta = (minT_daily + maxT_daily)/2

         # Saturation Vapor Pressure
        es_tmin = 0.6108 * np.exp((17.27 * minT_daily) / (minT_daily + 237.3))
        es_tmax = 0.6108 * np.exp((17.27 * maxT_daily) / (maxT_daily + 237.3))
        es = (es_tmin + es_tmax) / 2.0

        # Actual Vapor Pressure
        ea = rel_humidity * es
        # when relative humidity data is not available, we can use following approximation based min temperature
        #ea = 0.611 * np.exp((17.27 * minT_daily)/(minT_daily + 237.3))

        # Latitude in Radian
        lat_rad = latitude * np.pi/180

        # Julien Days
        doy = np.arange(cycle_begin, cycle_end+1)
        sd = 0.4093 * np.sin((2*np.pi*doy/365)-1.39)

        # Sunset Hour angle
        sha = np.arccos(-np.tan(lat_rad)*np.tan(sd))

        # Relative distance Earth to Sun
        dr = 1 + 0.033*np.cos(2*np.pi*doy/365)

        # Extraterrestrial radiation (top of atmosphere radiation)
        Ra = 0.082*(24*60/np.pi)*dr*((sha*np.sin(lat_rad)*np.sin(sd))+(np.cos(lat_rad)*np.cos(sd)*np.sin(sha)))

        # Clear-sky solar radiation
        Rso= Ra*(0.75+(0.00002*alt))

        # Net incoming Short-wave radiation
        alpha = 0.23
        Rns = shortRad_daily * (1-alpha)

        # Net outgoing long wave-radiation
        sub_cst = 0.000000004903  # Stefan-Boltzmann constant [MJ K-4 m-2 day-1]
        Rnl = (((273.16+maxT_daily)**4)+((273.16 +minT_daily)**4))*(0.34 - (0.14*(ea**0.5)))*((1.35*(shortRad_daily/Rso))-0.35)*sub_cst/2

        # Net Radiation flux at crop surface
        Rn = Rns - Rnl

        # Slope of Vapour Presure
        D = 4098 * (0.6108*np.exp((17.27*ta) /(237.3+ta)))/(ta+237.3)**2

        # Atmospheric Pressure
        P = 101.3*np.power(((293-(0.0065*alt))/293),5.26)

        # Psychrometric Constant
        psy = 0.000665*P

        # Soil Heat Flux - G
        ta_dublicate_last2 = np.append(ta, np.array([ta[-1], ta[-1]]))
        ta_dublicate_first2 = np.append(np.array([ta[0], ta[0]]), ta)
        G = 0.14 * (ta_dublicate_last2 - ta_dublicate_first2)
        G = G[0:G.size-2]

        # ETo Penman Monteith
        ETo_a = 0.408*D*(Rn-G)
        ETo_b = psy*900*windspeed_daily*(es-ea)/(ta+273)
        ETo_c = D + (psy*(1+0.34*windspeed_daily))
        ETo = (ETo_a + ETo_b)/ETo_c

        ETo[ETo < 0]= 0

        return ETo

    def calculateETO(self):
        return ETOCalc.calculateETONumba(self.cycle_begin, self.cycle_end, self.latitude, self.alt,  self.minT_daily, self.maxT_daily, self.windspeed_daily, self.shortRad_daily, self.rel_humidity)




"""
PyAEZ
Written by Thaileng Thol

BETA version - under development and in testing phase
"""

import numpy as np
from scipy import stats

class EconomicSuitability(object):

    def __init__(self):
        self.crop_name_list = []
        self.net_revenue_list = []
        self.revenue_min_list = []

    def addACrop(self, crop_name, crop_cost, crop_yield, farm_price, yield_map):

        slope, intercept, r_value1, p_value, std_err = stats.linregress(crop_yield, crop_cost)

        '''Minimum attainable yield Calculation- break-even analysis'''
        y_min = intercept/(np.mean(farm_price)-slope)


        '''Attainable net revenue'''
        net_revenue = (yield_map - y_min) * np.mean(farm_price)
        net_revenue[net_revenue<0] = 0

        self.crop_name_list.append(crop_name)
        self.net_revenue_list.append(net_revenue)
        self.revenue_min_list.append(y_min * np.mean(farm_price))

        return None

    def getNetRevenue(self, crop_name):
        return self.net_revenue_list[self.crop_name_list.index(crop_name)]

    def getClassifiedNetRevenue(self, crop_name):

        ''' Classification of Net Attainable Revenue map
        class 7 = very high = yields are more of the overall maximum yield,
        class 6 = high = yields between 63% and 75%,
        class 5 = good = yields between 50% and 63%,
        class 4 = medium = yields between 35% and 50%,
        class 3 = moderate = yields between 20% and 35%,
        class 2 = marginal  = yields between 10% and 20%,
        class 1 = very marginal = yields between 0% and 10%,
        class 0 = not suitable = yields less than 0%.
        '''

        net_revenue = self.net_revenue_list[self.crop_name_list.index(crop_name)]
        net_revenue_class = np.zeros(net_revenue.shape)

        if len(net_revenue[net_revenue>0]) > 0:
            net_revenue_max = np.amax(net_revenue[net_revenue>0])
            net_revenue_min = np.amin(net_revenue[net_revenue>0])
        else:
            net_revenue_max = 0
            net_revenue_min = 0
        
        net_revenue_10P = (net_revenue_max-net_revenue_min)*(10/100) + net_revenue_min
        net_revenue_20P = (net_revenue_max-net_revenue_min)*(20/100) + net_revenue_min
        net_revenue_35P = (net_revenue_max-net_revenue_min)*(35/100) + net_revenue_min
        net_revenue_50P = (net_revenue_max-net_revenue_min)*(50/100) + net_revenue_min
        net_revenue_63P = (net_revenue_max-net_revenue_min)*(63/100) + net_revenue_min
        net_revenue_75P = (net_revenue_max-net_revenue_min)*(75/100) + net_revenue_min

        net_revenue_class[ np.all([net_revenue<=0], axis=0) ] = 0 # not suitable
        net_revenue_class[ np.all([0<net_revenue, net_revenue<=net_revenue_10P], axis=0) ] = 1 # ver marginal
        net_revenue_class[ np.all([net_revenue_10P<net_revenue, net_revenue<=net_revenue_20P], axis=0) ] = 2 # marginal
        net_revenue_class[ np.all([net_revenue_20P<net_revenue, net_revenue<=net_revenue_35P], axis=0) ] = 3 # moderate
        net_revenue_class[ np.all([net_revenue_35P<net_revenue, net_revenue<=net_revenue_50P], axis=0) ] = 4 # medium
        net_revenue_class[ np.all([net_revenue_50P<net_revenue, net_revenue<=net_revenue_63P], axis=0) ] = 5 # good
        net_revenue_class[ np.all([net_revenue_63P<net_revenue, net_revenue<=net_revenue_75P], axis=0) ] = 6 # high
        net_revenue_class[ np.all([net_revenue_75P<net_revenue], axis=0)] = 7 # very high

        return net_revenue_class

    def getNormalizedNetRevenue(self, crop_name):

        net_revenue = self.net_revenue_list[self.crop_name_list.index(crop_name)]
        net_revenue_max =  np.max(np.array(self.net_revenue_list), axis=0)

        net_revenue_norm = np.zeros(net_revenue.shape)

        net_revenue_norm[net_revenue_max>0] = np.divide(net_revenue[net_revenue_max>0], net_revenue_max[net_revenue_max>0])

        net_revenue_norm_class = np.zeros(net_revenue.shape)

        net_revenue_norm_class[ np.all([net_revenue_norm<=0], axis=0) ] = 0 # not suitable
        net_revenue_norm_class[ np.all([0<net_revenue_norm, net_revenue_norm<=0.2], axis=0) ] = 1 #  marginal
        net_revenue_norm_class[ np.all([0.2<net_revenue_norm, net_revenue_norm<=0.4], axis=0) ] = 2 # moderate
        net_revenue_norm_class[ np.all([0.4<net_revenue_norm, net_revenue_norm<=0.6], axis=0) ] = 3 # fair
        net_revenue_norm_class[ np.all([0.6<net_revenue_norm, net_revenue_norm<=0.8], axis=0) ] = 4 # good
        net_revenue_norm_class[ np.all([0.8<net_revenue_norm, net_revenue_norm<=1.0], axis=0) ] = 5 # good

        return net_revenue_norm_class



"""
PyAEZ
Written by N. Lakmal Deshapriya
"""

import numpy as np
import csv

import ALL_REDUCTION_FACTORS_IRR as crop_P_IRR
import ALL_REDUCTION_FACTORS_RAIN as crop_P_RAIN

class SoilConstraints(object):

    '''functions for soil qualities'''

    def soil_qty_1_top(self, TXT_val, OC_val, pH_val, TEB_val):

        TXT_intp = self.crop_P.TXT1_factor[self.crop_P.TXT1_value.index(TXT_val)]/100
        pH_intp = np.interp(pH_val, self.crop_P.pH_value, self.crop_P.pH_factor)/100
        OC_intp = np.interp(OC_val, self.crop_P.OC_value, self.crop_P.OC_factor)/100
        TEB_intp = np.interp(TEB_val, self.crop_P.TEB_value, self.crop_P.TEB_factor)/100

        min_factor = np.min([TXT_intp, pH_intp, OC_intp, TEB_intp])
        final_factor = (min_factor + (np.sum([TXT_intp, pH_intp, OC_intp, TEB_intp]) - min_factor)/3)/2

        return final_factor

    def soil_qty_1_sub(self, TXT_val, pH_val, TEB_val):

        TXT_intp = self.crop_P.TXT1_factor[self.crop_P.TXT1_value.index(TXT_val)]/100
        pH_intp = np.interp(pH_val, self.crop_P.pH_value, self.crop_P.pH_factor)/100
        TEB_intp = np.interp(TEB_val, self.crop_P.TEB_value, self.crop_P.TEB_factor)/100

        min_factor = np.min([TXT_intp, pH_intp, TEB_intp])
        final_factor = (min_factor + (np.sum([TXT_intp, pH_intp, TEB_intp]) - min_factor)/2)/2

        return final_factor

    def soil_qty_2_top(self, TXT_val, BS_val, CECsoil_val):

        TXT_intp = self.crop_P.TXT2_factor[self.crop_P.TXT2_value.index(TXT_val)]/100
        BS_intp = np.interp(BS_val, self.crop_P.BS_value, self.crop_P.BS_factor)/100
        CECsoil_intp = np.interp(CECsoil_val, self.crop_P.CECsoil_value, self.crop_P.CECsoil_factor)/100

        min_factor = np.min([TXT_intp, BS_intp, CECsoil_intp])
        final_factor = (min_factor + (np.sum([TXT_intp, BS_intp, CECsoil_intp]) - min_factor)/2)/2

        return final_factor

    def soil_qty_2_sub(self, TXT_val, BS_val, CECclay_val, pH_val):

        TXT_intp = self.crop_P.TXT2_factor[self.crop_P.TXT2_value.index(TXT_val)]/100
        BS_intp = np.interp(BS_val, self.crop_P.BS_value, self.crop_P.BS_factor)/100
        CECclay_intp = np.interp(CECclay_val, self.crop_P.CECclay_value, self.crop_P.CECclay_factor)/100
        pH_intp = np.interp(pH_val, self.crop_P.pH_value, self.crop_P.pH_factor)/100

        min_factor = np.min([TXT_intp, BS_intp, CECclay_intp, pH_intp])
        final_factor = (min_factor + (np.sum([TXT_intp, BS_intp, CECclay_intp, pH_intp]) - min_factor)/3)/2

        return final_factor

    def soil_qty_3_top_sub(self, RSD_val, SPR_val, SPH_val, OSD_val):

        RSD_intp = np.interp(RSD_val, self.crop_P.RSD_value, self.crop_P.RSD_factor)/100
        SPR_intp = np.interp(SPR_val, self.crop_P.SPR_value, self.crop_P.SPR_factor)/100
        SPH_intp = self.crop_P.SPH3_factor[self.crop_P.SPH3_value.index(SPH_val)]/100
        OSD_intp = np.interp(OSD_val, self.crop_P.OSD_value, self.crop_P.OSD_factor)/100

        final_factor = RSD_intp * np.min([SPR_intp, SPH_intp, OSD_intp])

        return final_factor

    def soil_qty_4_top_sub(self, DRG_val, SPH_val):

        DRG_intp = self.crop_P.DRG_factor[self.crop_P.DRG_value.index(DRG_val)]/100
        SPH_intp = self.crop_P.SPH4_factor[self.crop_P.SPH4_value.index(SPH_val)]/100

        final_factor = np.min([DRG_intp, SPH_intp])

        return final_factor

    def soil_qty_5_top_sub(self, ESP_val, EC_val, SPH_val):

        ESP_intp = np.interp(ESP_val, self.crop_P.ESP_value, self.crop_P.ESP_factor)/100
        EC_intp = np.interp(EC_val, self.crop_P.EC_value, self.crop_P.EC_factor)/100
        SPH_intp = self.crop_P.SPH5_factor[self.crop_P.SPH5_value.index(SPH_val)]/100

        final_factor = np.min([ESP_intp*EC_intp, SPH_intp])

        return final_factor

    def soil_qty_6_top_sub(self, CCB_val, GYP_val, SPH_val):

        CCB_intp = np.interp(CCB_val, self.crop_P.CCB_value, self.crop_P.CCB_factor)/100
        GYP_intp = np.interp(GYP_val, self.crop_P.GYP_value, self.crop_P.GYP_factor)/100
        SPH_intp = self.crop_P.SPH6_factor[self.crop_P.SPH6_value.index(SPH_val)]/100

        final_factor = np.min([CCB_intp*GYP_intp, SPH_intp])

        return final_factor

    def soil_qty_7_top_sub(self, RSD_val, GRC_val, SPH_val, TXT_val, VSP_val):

        RSD_intp = np.interp(RSD_val, self.crop_P.RSD_value, self.crop_P.RSD_factor)/100
        GRC_intp = np.interp(GRC_val, self.crop_P.GRC_value, self.crop_P.GRC_factor)/100
        SPH_intp = self.crop_P.SPH7_factor[self.crop_P.SPH7_value.index(SPH_val)]/100
        TXT_intp = self.crop_P.TXT7_factor[self.crop_P.TXT7_value.index(TXT_val)]/100
        VSP_intp = np.interp(VSP_val, self.crop_P.VSP_value, self.crop_P.VSP_factor)/100

        min_factor = np.min([RSD_intp, GRC_intp, SPH_intp, TXT_intp, VSP_intp])
        final_factor = (min_factor + (np.sum([RSD_intp, GRC_intp, SPH_intp, TXT_intp, VSP_intp]) - min_factor)/4)/2

        return final_factor

    def calculateSoilQualities(self, irr_or_rain, topsoil_path='./sample_data/input/soil_characteristics_topsoil.csv', subsoil_path='./sample_data/input/soil_characteristics_subsoil.csv'):

        if irr_or_rain == 'I':
            self.crop_P = crop_P_IRR
        elif irr_or_rain == 'R':
            self.crop_P = crop_P_RAIN

        # getting number of soil units
        file_temp = open(topsoil_path)
        numline = len(file_temp.readlines())

        self.sq_mat = np.zeros((numline-1,1+7)) # first column for soil unit code and other columns for SQs

        with open(topsoil_path) as csv_top:
            csv_sub = open(subsoil_path)
            top_reader = csv.reader(csv_top, delimiter=',')
            sub_reader = csv.reader(csv_sub, delimiter=',')

            row_count = 0
            name_list = []
            for row_top in top_reader:
                row_sub = next(sub_reader)

                row_count = row_count + 1
                if row_count == 1:
                    name_list = row_top
                    continue

                self.sq_mat[row_count-2, 0] = float(row_top[0])

                '''SQ 1'''

                TXT_val = row_top[name_list.index('TXT')]
                OC_val = float(row_top[name_list.index('OC')])
                pH_val = float(row_top[name_list.index('pH')])
                TEB_val = float(row_top[name_list.index('TEB')])
                sq1_top = self.soil_qty_1_top(TXT_val, OC_val, pH_val, TEB_val)

                TXT_val = row_sub[name_list.index('TXT')]
                pH_val = float(row_sub[name_list.index('pH')])
                TEB_val = float(row_sub[name_list.index('TEB')])
                sq1_sub = self.soil_qty_1_sub(TXT_val, pH_val, TEB_val)

                sq1 = (sq1_top + sq1_sub)/2

                self.sq_mat[row_count-2, 1] = sq1

                '''SQ 2'''

                TXT_val = row_top[name_list.index('TXT')]
                BS_val = float(row_top[name_list.index('BS')])
                CECsoil_val = float(row_top[name_list.index('CEC_soil')])
                sq2_top = self.soil_qty_2_top(TXT_val, BS_val, CECsoil_val)

                TXT_val = row_sub[name_list.index('TXT')]
                BS_val = float(row_sub[name_list.index('BS')])
                CECclay_val = float(row_sub[name_list.index('CEC_soil')])
                pH_val = float(row_sub[name_list.index('pH')])
                sq2_sub = self.soil_qty_2_sub(TXT_val, BS_val, CECclay_val, pH_val)

                sq2 = (sq2_top + sq2_sub)/2

                self.sq_mat[row_count-2, 2] = sq2

                '''SQ 3'''

                RSD_val = float(row_top[name_list.index('RSD')])
                SPR_val = float(row_top[name_list.index('SPR')])
                SPH_val = row_top[name_list.index('SPH')]
                OSD_val = float(row_top[name_list.index('OSD')])
                sq3_top = self.soil_qty_3_top_sub(RSD_val, SPR_val, SPH_val, OSD_val)

                RSD_val = float(row_sub[name_list.index('RSD')])
                SPR_val = float(row_sub[name_list.index('SPR')])
                SPH_val = row_sub[name_list.index('SPH')]
                OSD_val = float(row_sub[name_list.index('OSD')])
                sq3_sub = self.soil_qty_3_top_sub(RSD_val, SPR_val, SPH_val, OSD_val)

                sq3 = (sq3_top + sq3_sub)/2

                self.sq_mat[row_count-2, 3] = sq3

                '''SQ 4'''

                DRG_val = row_top[name_list.index('DRG')]
                SPH_val = row_top[name_list.index('SPH')]
                sq4_top = self.soil_qty_4_top_sub(DRG_val, SPH_val)

                DRG_val = row_sub[name_list.index('DRG')]
                SPH_val = row_sub[name_list.index('SPH')]
                sq4_sub = self.soil_qty_4_top_sub(DRG_val, SPH_val)

                sq4 = (sq4_top + sq4_sub)/2

                self.sq_mat[row_count-2, 4] = sq4

                '''SQ 5'''

                ESP_val = float(row_top[name_list.index('ESP')])
                EC_val = float(row_top[name_list.index('EC')])
                SPH_val = row_top[name_list.index('SPH')]
                sq5_top = self.soil_qty_5_top_sub(ESP_val, EC_val, SPH_val)

                ESP_val = float(row_sub[name_list.index('ESP')])
                EC_val = float(row_sub[name_list.index('EC')])
                SPH_val = row_sub[name_list.index('SPH')]
                sq5_sub = self.soil_qty_5_top_sub(ESP_val, EC_val, SPH_val)

                sq5 = (sq5_top + sq5_sub)/2

                self.sq_mat[row_count-2, 5] = sq5

                '''SQ 6'''

                CCB_val = float(row_top[name_list.index('CCB')])
                GYP_val = float(row_top[name_list.index('GYP')])
                SPH_val = row_top[name_list.index('SPH')]
                sq6_top = self.soil_qty_6_top_sub(CCB_val, GYP_val, SPH_val)

                CCB_val = float(row_sub[name_list.index('CCB')])
                GYP_val = float(row_sub[name_list.index('GYP')])
                SPH_val = row_sub[name_list.index('SPH')]
                sq6_sub = self.soil_qty_6_top_sub(CCB_val, GYP_val, SPH_val)

                sq6 = (sq6_top + sq6_sub)/2

                self.sq_mat[row_count-2, 6] = sq6

                '''SQ 7'''

                RSD_val = float(row_top[name_list.index('RSD')])
                GRC_val = float(row_top[name_list.index('GRC')])
                SPH_val = row_top[name_list.index('SPH')]
                TXT_val = row_top[name_list.index('TXT')]
                VSP_val = float(row_top[name_list.index('VSP')])
                sq7_top = self.soil_qty_7_top_sub(RSD_val, GRC_val, SPH_val, TXT_val, VSP_val)

                RSD_val = float(row_sub[name_list.index('RSD')])
                GRC_val = float(row_sub[name_list.index('GRC')])
                SPH_val = row_sub[name_list.index('SPH')]
                TXT_val = row_sub[name_list.index('TXT')]
                VSP_val = float(row_sub[name_list.index('VSP')])
                sq7_sub = self.soil_qty_7_top_sub(RSD_val, GRC_val, SPH_val, TXT_val, VSP_val)

                sq7 = (sq7_top + sq7_sub)/2

                self.sq_mat[row_count-2, 7] = sq7

    def calculateSoilRatings(self, input_level):

        self.SR = np.zeros((self.sq_mat.shape[0],2)) # first column for soil unit code and other column for SR
        self.SR[:,0] = self.sq_mat[:,0] # adding soil unit code

        for i1 in range(0, self.sq_mat.shape[0]):

            if input_level == 'L':
                min_factor = np.min([self.sq_mat[i1][4], self.sq_mat[i1][5], self.sq_mat[i1][6], self.sq_mat[i1][7]])
                fsq = (min_factor + (np.sum([self.sq_mat[i1][4], self.sq_mat[i1][5], self.sq_mat[i1][6], self.sq_mat[i1][7]]) - min_factor)/3)/2
                self.SR[i1,1] = self.sq_mat[i1][1] * self.sq_mat[i1][3] * fsq
            elif input_level == 'I':
                min_factor = np.min([self.sq_mat[i1][4], self.sq_mat[i1][5], self.sq_mat[i1][6], self.sq_mat[i1][7]])
                fsq = (min_factor + (np.sum([self.sq_mat[i1][4], self.sq_mat[i1][5], self.sq_mat[i1][6], self.sq_mat[i1][7]]) - min_factor)/3)/2
                self.SR[i1,1] = 0.5 * (self.sq_mat[i1][1]+self.sq_mat[i1][2]) * self.sq_mat[i1][3] * fsq
            elif input_level == 'H':
                min_factor = np.min([self.sq_mat[i1][4], self.sq_mat[i1][5], self.sq_mat[i1][6], self.sq_mat[i1][7]])
                fsq = (min_factor + (np.sum([self.sq_mat[i1][4], self.sq_mat[i1][5], self.sq_mat[i1][6], self.sq_mat[i1][7]]) - min_factor)/3)/2
                self.SR[i1,1] = self.sq_mat[i1][2] * self.sq_mat[i1][3] * fsq
            else:
                print('Wrong Input Level !')

    def getSoilQualities(self):
        return self.sq_mat

    def getSoilRatings(self):
        return self.SR

    def applySoilConstraints(self, soil_map, yield_in):

        yield_final = np.copy(yield_in)

        for i1 in range(0, self.SR.shape[0]):
            temp_idx = soil_map==self.SR[i1,0]
            yield_final[temp_idx] = yield_in[temp_idx] * self.SR[i1,1]

        return yield_final



import os
from typing import Dict

from setuptools import setup, find_packages

HERE = os.path.abspath(os.path.dirname(__file__))

about = dict()

with open(os.path.join(HERE, "pyaez", "__version__.py"), "r") as f:
    exec(f.read(), about)

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name="PyAEZ",
    version=about["__version__"],
    author=about["__author__"],
    author_email=about["__email__"],
    description="PyAEZ is a python package consisted of many algorithms related to Agro-ecalogical zoning (AEZ) framework.",
    py_modules=["pyaez"],
    # package_dir={'':'src'},
    license="MIT License",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/gicait/PyAEZ",
    packages=find_packages(),
    keywords=[
        "AEZ",
        "Python",
        "Agro-ecological Zoning",
        "Climate Regime",
        "Crop Simulations",
        "Climate Constraints",
        "Soil Constraints",
        "Terrain Constraints",
        "Economic Suitability Analysis",
        "Utility Calculation",
        "Biomass Calculations",
        "Evapotranspiration Calculation",
        "CropWat calculations",
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "Intended Audience :: Developers",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=[
        'numpy',
        'scipy',
        'gdal',
        'numba',
    ],
    extras_require={"dev": [
        "pytest",
        "black",
        "flake8",
        "sphinx>=1.7",
        "pre-commit"
    ]},
    python_requires=">=3.6",
)


"""
PyAEZ
Written by N. Lakmal Deshapriya and Thaileng Thol

Reference: http://oar.icrisat.org/198/1/316_2009_GTAE_55_Poten_obt_yield_in_SAT.pdf
"""

import numpy as np
import numba as nb

class CropWatCalc(object):

    def __init__(self, cycle_begin, cycle_end):
        self.cycle_begin = cycle_begin
        self.cycle_end = cycle_end
        self.cycle_len = cycle_end - cycle_begin + 1

    def setClimateData(self, pet, precipitation):
        self.peto = pet
        self.Prec = precipitation

    def setCropParameters(self, stage_per, kc, kc_all, yloss_f, yloss_f_all, est_yield, D1, D2, Sa, pc):
        self.d_per = np.array(stage_per) # Percentage for D1, D2, D3, D4 stages
        self.kc = np.array(kc) # 3 crop water requirements for initial, reproductive, the end of the maturation stages
        self.kc_all = kc_all # crop water requirements for entire growth cycle
        self.yloss_f = np.array(yloss_f)  # yield loss for D1, D2, D3, D4
        self.yloss_f_all = yloss_f_all  # yield loss for entire growth cycle
        self.y_potential = est_yield  # potential yield
        self.D1 = D1  # rooting depth (m)
        self.D2 = D2  # rooting depth (m)
        self.Sa = Sa  # available soil moisture holding capacity (mm/m) , usually assume as 100
        self.pc = pc  # soil water depletion fraction below which ETa < ETo (from literature)

    @staticmethod
    @nb.jit(nopython=True)
    def calculateMoistureLimitedYieldNumba(cycle_begin, cycle_end, cycle_len, peto, Prec, d_per, kc, kc_all, yloss_f, yloss_f_all, y_potential, D1, D2, Sa, pc):
        '''Convert Percentage of stage in the crop cycle to cumulative number of days'''

        d_days = (cycle_len * (np.cumsum(d_per)/100)).astype(np.int_)
        #d_days = np.round( cycle_len * (np.cumsum(d_per)/100) ).astype(np.int_)

        '''Interpolate Rotting Depth (D)'''
        D = np.zeros(peto.size)
        D[d_days[1]:] = D2
        D[:d_days[1]] = D1 + ((D2-D1)/(d_days[1])) * np.arange(d_days[1])

        '''Lets calculate crop specific PET for each stage'''

        petc_stage = np.zeros(peto.size)
        petc_all = np.zeros(peto.size)

        for ii in range(cycle_len):

            ## each growing stage
            if ii <= d_days[0]:
                petc_stage[ii] = kc[0] * peto[ii]
            elif ii <= d_days[1]:
                kc_temp = kc[0] + (ii-d_days[0]) * ( (kc[1]-kc[0])/(d_days[1]-d_days[0]) )
                petc_stage[ii] = kc_temp * peto[ii]
            elif ii <= d_days[2]:
                petc_stage[ii] = kc[1] * peto[ii]
            else:
                kc_temp = kc[1] + (ii-d_days[2]) * ( (kc[2]-kc[1])/(d_days[3]-d_days[2]) )
                petc_stage[ii] = kc_temp * peto[ii]

            ## entire gorwing season
            petc_all[ii] = kc_all * peto[ii]

        '''Lets calculate Actual ET for each stage'''

        peta_stage = np.zeros(peto.size)
        peta_all = np.zeros(peto.size)

        ## each growing stage
        W = Sa*D[0]
        for ii in range(cycle_len):

            if Prec[ii] >= petc_stage[ii]:
                peta_stage[ii] = petc_stage[ii]
            elif Prec[ii] + W >= Sa*D[ii]*(1-pc):
                peta_stage[ii] = petc_stage[ii]
            else:
                kk = (W+Prec[ii]) / (Sa*D[ii]*(1-pc))
                peta_stage[ii] = kk * petc_stage[ii]

            W = np.min(np.array([W+Prec[ii]-peta_stage[ii], Sa*D[ii]]))
            if W<0: W=0

        ## entire gorwing season
        W = Sa*D[0]
        for ii in range(cycle_len):

            if Prec[ii] >= petc_all[ii]:
                peta_all[ii] = petc_all[ii]
            elif Prec[ii] + W >= Sa*D[ii]*(1-pc):
                peta_all[ii] = petc_all[ii]
            else:
                kk = (W+Prec[ii]) / (Sa*D[ii]*(1-pc))
                peta_all[ii] = kk * petc_all[ii]

            W = np.min(np.array([W+Prec[ii]-peta_all[ii], Sa*D[ii]]))
            if W<0: W=0

        '''Assess Yield Loss in entire growth cycle'''

        peta_all_sum = np.sum( peta_all )
        petc_all_sum = np.sum( petc_all )

        f0 = 1 - yloss_f_all * ( 1 - (peta_all_sum/petc_all_sum) )

        '''Assess Yield Loss in individual growth stages separately'''

        peta_d1 = np.sum( peta_stage[0:d_days[0]] )
        peta_d2 = np.sum( peta_stage[d_days[0]:d_days[1]] )
        peta_d3 = np.sum( peta_stage[d_days[1]:d_days[2]] )
        peta_d4 = np.sum( peta_stage[d_days[2]:d_days[3]] )

        petc_d1 = np.sum( petc_stage[0:d_days[0]] )
        petc_d2 = np.sum( petc_stage[d_days[0]:d_days[1]] )
        petc_d3 = np.sum( petc_stage[d_days[1]:d_days[2]] )
        petc_d4 = np.sum( petc_stage[d_days[2]:d_days[3]] )

        f1_d1 = 1 - yloss_f[0] * ( 1 - (peta_d1/petc_d1) )
        f1_d2 = 1 - yloss_f[1] * ( 1 - (peta_d2/petc_d2) )
        f1_d3 = 1 - yloss_f[2] * ( 1 - (peta_d3/petc_d3) )
        f1_d4 = 1 - yloss_f[3] * ( 1 - (peta_d4/petc_d4) )

        f1 = np.min(np.array([f1_d1,f1_d2,f1_d3,f1_d4])) # some references use product, some use minimum. here we use minimum as in Thailand report

        '''Use more severe of above two conditions determines final yield'''

        f_final = np.min(np.array([f0,f1]))

        # to avoid, possible error
        if f_final < 0:
            f_final = 0
        if f_final > 1:
            f_final = 1

        y_water_limited = f_final * y_potential

        return y_water_limited

    def calculateMoistureLimitedYield(self):
        return CropWatCalc.calculateMoistureLimitedYieldNumba(self.cycle_begin, self.cycle_end, self.cycle_len, self.peto, self.Prec, self.d_per, self.kc, self.kc_all, self.yloss_f, self.yloss_f_all, self.y_potential, self.D1, self.D2, self.Sa, self.pc)



"""
PyAEZ
Written by N. Lakmal Deshapriya
"""

import numpy as np

class BioMassCalc(object):

    '''Max Radiation'''
    Ac = np.array([[343,360,369,364,349,337,342,357,368,365,349,337],
        [299,332,359,375,377,374,375,377,369,345,311,291],
        [249,293,337,375,394,400,399,386,357,313,264,238],
        [191,245,303,363,400,417,411,384,333,270,210,179],
        [131,190,260,339,396,422,413,369,298,220,151,118],
         [73,131,207,304,380,418,405,344,254,163, 92, 61],
         [22, 72,149,260,356,408,389,309,201,103, 37, 14],
          [0, 20, 89,209,331,408,380,269,142, 45,  2,  0],
          [0,  0, 28,162,334,424,393,248, 81,  3,  0,  0],
          [0,  0,  0,154,339,428,397,252, 40,  0,  0,  0]]);

    '''Biomass in open day'''
    bc = np.array([[413,424,429,426,417,410,413,422,429,427,418,410],
        [376,401,422,437,440,440,440,439,431,411,385,370],
        [334,371,407,439,460,468,465,451,425,387,348,325],
        [281,333,385,437,471,489,483,456,412,356,299,269],
        [218,283,353,427,480,506,497,455,390,314,241,204],
        [147,223,310,409,484,522,509,488,358,260,173,130],
         [66,151,254,383,487,544,523,436,316,195, 94, 49],
          [0, 65,185,350,506,612,575,427,262,114,  7,  0],
          [0,  0, 94,333,571,663,632,474,195, 11,  0,  0],
          [0,  0,  0,371,588, 67,646,497,167,  0,  0,  0]]);

    ''' Biomass in cloudy day '''
    bo = np.array([[219,226,230,228,221,216,218,225,230,228,222,216],
        [197,212,225,234,236,235,236,235,230,218,203,193],
        [170,193,215,235,246,250,249,242,226,203,178,164],
        [137,168,200,232,251,261,258,243,216,182,148,130],
         [99,137,178,223,253,268,263,239,200,155,112, 91],
         [60,100,150,207,251,273,265,230,178,121, 73, 51],
         [19, 60,114,187,245,276,265,216,148, 82, 31, 11],
          [0, 16, 74,158,241,291,273,200,112, 38,  1,  0],
          [0,  0, 24,133,257,318,297,196, 69,  2,  0,  0],
          [0,  0,  0,131,269,319,302,215, 35,  0,  0,  0]]);

    '''Pm values table'''
    PmIndexExtDtTemp = np.array([5,10,15,20,25,30,35]);
    PmIndexExt = np.array([[0,15,20,20,  15, 5, 0],
        [0, 0,15,32.5,35,35,35],
        [0, 0, 5,45,65,65,65],
        [0, 5,45,65, 65,65,65]]);

    def __init__(self, cycle_begin, cycle_end, latitude):
        self.cycle_begin = cycle_begin
        self.cycle_end = cycle_end
        self.cycle_len = cycle_end - cycle_begin + 1

        self.lat = np.abs(latitude)
        self.lat_index1 = np.floor(np.abs(latitude)/10);
        self.lat_index2 = np.ceil(np.abs(latitude)/10);

    def setClimateData(self, min_temp, max_temp, short_rad):

        self.minT_daily = min_temp
        self.maxT_daily = max_temp
        self.shortRad_daily = short_rad

        ## convert radiation values from W/m^2 to cal/cm2/day1
        self.shortRad_daily = ((self.shortRad_daily / 4.19) * (60*60*24)) / (100*100);

        '''Calculate Mean T and dT'''

        ## Calculate Mean Temperature Values
        self.meanT_daily = (self.minT_daily + self.maxT_daily)/2;

        ## Calculate Mean dT Values (day time temperature)
        self.dT_daily = np.zeros(self.minT_daily.shape);

        i_count = 0
        for i1 in range(self.cycle_begin, self.cycle_end+1):

            if i1<30:
                self.dT_daily[i_count] = 0.0278 + 0.3301*self.minT_daily[i_count] + 0.6716*self.maxT_daily[i_count];
            elif i1<30*2:
                self.dT_daily[i_count] = 0.078 + 0.3345*self.minT_daily[i_count] + 0.6672*self.maxT_daily[i_count];
            elif i1<30*3:
                self.dT_daily[i_count] = 0.0770 + 0.3392*self.minT_daily[i_count] + 0.6642*self.maxT_daily[i_count];
            elif i1<30*4:
                self.dT_daily[i_count] = 0.2276 + 0.3466*self.minT_daily[i_count] + 0.6536*self.maxT_daily[i_count];
            elif i1<30*5:
                self.dT_daily[i_count] = 0.2494 + 0.3399*self.minT_daily[i_count] + 0.6576*self.maxT_daily[i_count];
            elif i1<30*6:
                self.dT_daily[i_count] = 0.9955+ 0.3335*self.minT_daily[i_count] + 0.6628*self.maxT_daily[i_count];
            elif i1<30*7:
                self.dT_daily[i_count] = 0.2624+ 0.3351*self.minT_daily[i_count] + 0.659*self.maxT_daily[i_count];
            elif i1<30*8:
                self.dT_daily[i_count] = 0.2860+ 0.3297*self.minT_daily[i_count] + 0.6612*self.maxT_daily[i_count];
            elif i1<30*9:
                self.dT_daily[i_count] = 0.3492+ 0.3410*self.minT_daily[i_count] + 0.6515*self.maxT_daily[i_count];
            elif i1<30*10:
                self.dT_daily[i_count] = 0.1598+ 0.3394*self.minT_daily[i_count] + 0.6591*self.maxT_daily[i_count];
            elif i1<30*11:
                self.dT_daily[i_count] = 0.1156+ 0.3476*self.minT_daily[i_count] + 0.6571*self.maxT_daily[i_count];
            else:
                self.dT_daily[i_count] = -0.0617+ 0.3462*self.minT_daily[i_count] + 0.6649*self.maxT_daily[i_count];

            i_count = i_count + 1

    def setCropParameters(self, LAI, HI, legume, adaptability):
        self.LAi = LAI # leaf area index
        self.HI = HI # harvest index
        self.legume = legume # binary value
        self.adaptability = adaptability-1  #one of [1,2,3,4] classes

    def calculateBioMass(self):
        ''' Calculate average values of Ac, bc, bo, meanT, dT  in the season '''

        doy_middle_of_month = np.arange(0,12)*30 + 15 # Calculate doy of middle of month

        Ac_interp1 = np.interp(np.arange(self.cycle_begin, self.cycle_end+1), doy_middle_of_month, BioMassCalc.Ac[int(self.lat_index1),:])
        bc_interp1 = np.interp(np.arange(self.cycle_begin, self.cycle_end+1), doy_middle_of_month, BioMassCalc.bc[int(self.lat_index1),:])
        bo_interp1 = np.interp(np.arange(self.cycle_begin, self.cycle_end+1), doy_middle_of_month, BioMassCalc.bo[int(self.lat_index1),:])
        Ac_interp2 = np.interp(np.arange(self.cycle_begin, self.cycle_end+1), doy_middle_of_month, BioMassCalc.Ac[int(self.lat_index2),:])
        bc_interp2 = np.interp(np.arange(self.cycle_begin, self.cycle_end+1), doy_middle_of_month, BioMassCalc.bc[int(self.lat_index2),:])
        bo_interp2 = np.interp(np.arange(self.cycle_begin, self.cycle_end+1), doy_middle_of_month, BioMassCalc.bo[int(self.lat_index2),:])

        Ac_interp = Ac_interp1 + (int(self.lat)-int(self.lat_index1)*10)*((Ac_interp2-Ac_interp1)/10)
        bc_interp = bc_interp1 + (int(self.lat)-int(self.lat_index1)*10)*((bc_interp2-bc_interp1)/10)
        bo_interp = bo_interp1 + (int(self.lat)-int(self.lat_index1)*10)*((bo_interp2-bo_interp1)/10)

        self.Ac_mean = np.mean( Ac_interp );
        self.bc_mean = np.mean( bc_interp );
        self.bo_mean = np.mean( bo_interp );

        self.meanT_mean = np.mean( self.meanT_daily );
        self.dT_mean = np.mean( self.dT_daily );
        self.shortRad_mean = np.mean( self.shortRad_daily );

        '''the Fraction of the Daytime the Sky is Clouded'''

        Rg = self.shortRad_mean;
        f_day_clouded = (self.Ac_mean - (0.5 * Rg))/(0.8 * (self.Ac_mean));

        '''Maximum net Rate of CO 2 Exchange of Leaves'''

        PmIndexExt_1Row = BioMassCalc.PmIndexExt[self.adaptability,:];
        iPm = np.interp(self.dT_mean,BioMassCalc.PmIndexExtDtTemp,PmIndexExt_1Row);

        '''Adjust for Temperature and LAI'''
        # Calculate Ct
        if self.legume == 1:
            c = 0.0283;
        else:
            c = 0.0108;

        Ct = c*(0.0044 + 0.0019*self.meanT_mean + 0.0010*np.power(self.meanT_mean,2));

        if(1 <= self.LAi and self.LAi < 2):
            l = 0.4;
        elif(self.LAi < 3):
            l = 0.6;
        elif(self.LAi < 4):
            l = 0.8;
        elif(self.LAi < 5):
            l = 0.9;
        elif(5 <= self.LAi):
            l = 1;
        elif(LAi < 1):
            print("LAI too Low")
            return

        '''Maximum Rate of Gross Biomass Production'''

        if iPm > 20:
            bgm = f_day_clouded * (.8 + 0.01 * iPm) * self.bo_mean + (1-f_day_clouded) * (.5 + .025 * iPm) * self.bc_mean;
        elif iPm < 20:
            bgm = f_day_clouded * (.5 + .025 * iPm) * self.bo_mean + (1-f_day_clouded) * (.05 * iPm) * self.bc_mean;
        else:
            bgm = f_day_clouded*self.bo_mean + (1 - f_day_clouded)*self.bc_mean;

        '''net biomass production '''

        self.Bn = (0.36 * bgm * l) / ( (1/self.cycle_len) + 0.25*Ct );

        return self.Bn

    def calculateYield(self):
        self.PYield = self.Bn * self.HI;
        return self.PYield



"""
PyAEZ
Written by N. Lakmal Deshapriya
"""

import numpy as np

import ALL_REDUCTION_FACTORS_IRR as crop_P_IRR
import ALL_REDUCTION_FACTORS_RAIN as crop_P_RAIN

class TerrainConstraints(object):

    def setClimateTerrainData(self, precipitation, slope):
        self.prec_monthly = precipitation
        self.slope = slope # Percentage Slope

        self.im_height = slope.shape[0]
        self.im_width = slope.shape[1]

    def calculateFI(self):
        # calculation of Fournier index

        sum_Psquare = np.sum(np.square(self.prec_monthly), axis=2)
        sum_P = np.sum(self.prec_monthly, axis=2)

        self.FI = 12 * (sum_Psquare / sum_P)

    def getFI(self):
        # returning Fournier index

        return self.FI

    def applyTerrainConstraints(self, yield_in, irr_or_rain):

        if irr_or_rain == 'I':
            crop_P = crop_P_IRR
        elif irr_or_rain == 'R':
            crop_P = crop_P_RAIN

        yield_final = np.copy(yield_in)

        self.FI[np.isnan(self.FI)] = -99 # This suppresses warning with NaN values
        self.slope[np.isnan(self.slope)] = -99 # This suppresses warning with NaN values

        FI_count = -1
        for FI_cls1 in crop_P.FI_class:
            FI_count = FI_count + 1

            slope_count = -1
            for slope_cls1 in crop_P.Slope_class:
                slope_count = slope_count + 1

                FI_idx = np.logical_and(FI_cls1[0]<=self.FI, self.FI<=FI_cls1[1])
                slope_idx = np.logical_and(slope_cls1[0]<=self.slope, self.slope<=slope_cls1[1])
                temp_idx = np.logical_and(FI_idx, slope_idx)

                yield_final[temp_idx] = yield_in[temp_idx] * (crop_P.Terrain_factor[FI_count][slope_count] / 100)

        return yield_final



"""
PyAEZ
Written by N. Lakmal Deshapriya
"""

import numpy as np

class ThermalScreening(object):

    def __init__(self):
        self.set_tclimate_screening = False
        self.set_lgpt_screening = False
        self.set_Tsum_screening = False
        self.set_Tprofile_screening = False


    def getThermalLGP0(self):
        return np.sum(self.meanT_daily>0)

    def getThermalLGP5(self):
        return np.sum(self.meanT_daily>5)

    def getThermalLGP10(self):
        return np.sum(self.meanT_daily>10)

    def getTemperatureSum0(self):
        tempT = self.meanT_daily
        tempT[tempT<=0] = 0
        return np.sum(tempT)

    def getTemperatureSum5(self):
        tempT = self.meanT_daily
        tempT[tempT<=5] = 0
        return np.sum(tempT)

    def getTemperatureSum10(self):
        tempT = self.meanT_daily
        tempT[tempT<=10] = 0
        return np.sum(tempT)

    def getTemperatureProfile(self):

        meanT_daily_add1day = np.concatenate((self.meanT_daily, self.meanT_daily[0:1]))
        meanT_first = meanT_daily_add1day[:-1]
        meanT_diff = meanT_daily_add1day[1:] - meanT_daily_add1day[:-1]

        A9 = np.sum( np.logical_and(meanT_diff>0, meanT_first<-5) )
        A8 = np.sum( np.logical_and(meanT_diff>0, np.logical_and(meanT_first>=-5, meanT_first<0)) )
        A7 = np.sum( np.logical_and(meanT_diff>0, np.logical_and(meanT_first>=0, meanT_first<5)) )
        A6 = np.sum( np.logical_and(meanT_diff>0, np.logical_and(meanT_first>=5, meanT_first<10)) )
        A5 = np.sum( np.logical_and(meanT_diff>0, np.logical_and(meanT_first>=10, meanT_first<15)) )
        A4 = np.sum( np.logical_and(meanT_diff>0, np.logical_and(meanT_first>=15, meanT_first<20)) )
        A3 = np.sum( np.logical_and(meanT_diff>0, np.logical_and(meanT_first>=20, meanT_first<25)) )
        A2 = np.sum( np.logical_and(meanT_diff>0, np.logical_and(meanT_first>=25, meanT_first<30)) )
        A1 = np.sum( np.logical_and(meanT_diff>0, meanT_first>=30) )

        B9 = np.sum( np.logical_and(meanT_diff<0, meanT_first<-5) )
        B8 = np.sum( np.logical_and(meanT_diff<0, np.logical_and(meanT_first>=-5, meanT_first<0)) )
        B7 = np.sum( np.logical_and(meanT_diff<0, np.logical_and(meanT_first>=0, meanT_first<5)) )
        B6 = np.sum( np.logical_and(meanT_diff<0, np.logical_and(meanT_first>=5, meanT_first<10)) )
        B5 = np.sum( np.logical_and(meanT_diff<0, np.logical_and(meanT_first>=10, meanT_first<15)) )
        B4 = np.sum( np.logical_and(meanT_diff<0, np.logical_and(meanT_first>=15, meanT_first<20)) )
        B3 = np.sum( np.logical_and(meanT_diff<0, np.logical_and(meanT_first>=20, meanT_first<25)) )
        B2 = np.sum( np.logical_and(meanT_diff<0, np.logical_and(meanT_first>=25, meanT_first<30)) )
        B1 = np.sum( np.logical_and(meanT_diff<0, meanT_first>=30) )

        return [A9,A8,A7,A6,A5,A4,A3,A2,A1,B1,B2,B3,B4,B5,B6,B7,B8,B9]



    def setClimateData(self, minT_daily, maxT_daily):
        self.meanT_daily = (minT_daily + maxT_daily) / 2

        self.lgp0 = self.getThermalLGP0()
        self.lgp5 = self.getThermalLGP5()
        self.lgp10 = self.getThermalLGP10()

        self.tsum0 = self.getTemperatureSum0()
        self.tsum5 = self.getTemperatureSum5()
        self.tsum10 = self.getTemperatureSum10()

        self.tprofile = self.getTemperatureProfile()

    def setThermalClimateScreening(self, t_climate, no_t_climate):
        self.t_climate = t_climate
        self.no_t_climate = no_t_climate # list of unsuitable thermal climate

        self.set_tclimate_screening = True

    def setLGPTScreening(self, no_lgpt, optm_lgpt):
        self.no_lgpt = no_lgpt
        self.optm_lgpt = optm_lgpt

        self.set_lgpt_screening = True

    def setTSumScreening(self, no_Tsum, optm_Tsum):
        self.no_Tsum = no_Tsum
        self.optm_Tsum = optm_Tsum

        self.set_Tsum_screening = True

    def setTProfileScreening(self, no_Tprofile, optm_Tprofile):
        self.no_Tprofile = no_Tprofile
        self.optm_Tprofile = optm_Tprofile

        self.set_Tprofile_screening = True

    def getSuitability(self):

        if self.set_tclimate_screening:
            if self.t_climate in self.no_t_climate:
                return False

        if self.set_lgpt_screening:
            if self.lgp0<=self.no_lgpt[0] or self.lgp5<=self.no_lgpt[1] or self.lgp10<=self.no_lgpt[2]:
                return False

        if self.set_Tsum_screening:
            if self.tsum0<=self.no_Tsum[0] or self.tsum5<=self.no_Tsum[1] or self.tsum10<=self.no_Tsum[2]:
                return False


        if self.set_Tprofile_screening:
            for i1 in range(len(self.tprofile)):
                if self.tprofile[i1] <= self.no_Tprofile[i1]:
                    return False

        return True

    def getReductionFactor(self):

        thermal_screening_f = 1

        if self.set_lgpt_screening:

            if self.lgp0 < self.optm_lgpt[0]:
                f1 = ((self.lgp0-self.no_lgpt[0])/(self.optm_lgpt[0]-self.no_lgpt[0])) * 0.75 + 0.25
                thermal_screening_f = np.min([f1,thermal_screening_f])

            if self.lgp5 < self.optm_lgpt[1]:
                f1 = ((self.lgp5-self.no_lgpt[1])/(self.optm_lgpt[1]-self.no_lgpt[1])) * 0.75 + 0.25
                thermal_screening_f = np.min([f1,thermal_screening_f])

            if self.lgp10 < self.optm_lgpt[2]:
                f1 = ((self.lgp10-self.no_lgpt[2])/(self.optm_lgpt[2]-self.no_lgpt[2])) * 0.75 + 0.25
                thermal_screening_f = np.min([f1,thermal_screening_f])


        if self.set_Tsum_screening:

            if self.tsum0 < self.optm_Tsum[0]:
                f1 = ((self.tsum0-self.no_Tsum[0])/(self.optm_Tsum[0]-self.no_Tsum[0])) * 0.75 + 0.25
                thermal_screening_f = np.min([f1,thermal_screening_f])

            if self.tsum5 < self.optm_Tsum[1]:
                f1 = ((self.tsum5-self.no_Tsum[1])/(self.optm_Tsum[1]-self.no_Tsum[1])) * 0.75 + 0.25
                thermal_screening_f = np.min([f1,thermal_screening_f])

            if self.tsum10 < self.optm_Tsum[2]:
                f1 = ((self.tsum10-self.no_Tsum[2])/(self.optm_Tsum[2]-self.no_Tsum[2])) * 0.75 + 0.25
                thermal_screening_f = np.min([f1,thermal_screening_f])


        if self.set_Tprofile_screening:

            for i1 in range(len(tprofile)):
                if self.tprofile[i1] < self.optm_Tprofile[i1]:
                    f1 = ((self.tprofile[i1]-self.no_Tprofile[i1])/(self.optm_Tprofile[i1]-self.no_Tprofile[i1])) * 0.75 + 0.25
                    thermal_screening_f = np.min([f1,thermal_screening_f])

        return thermal_screening_f
